//
//  Duffing.m
//  Ex52
//
//  Created by 川上 博 on 10/04/22.
//  Copyright 2010 徳島大学. All rights reserved.
//

#import "Duffing.h"
#include <math.h>

#define PI 3.14159265359

double ak=0.2;
int poi=1;

@implementation Duffing

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		xm[0]=-2.0; xm[1]=4.0; ym[0]=-5.0; ym[1]=5.0;
		wm[0]=50.0; wm[1]=30.0; hm[0]=50.0; hm[1]=30.0;
		la[0]=0.5; la[1]=0.5; lav[0]=2; lav[1]=2;
		x[0]=1.2; x[1]=0.0;
		t=0.0; m=128; h=2.0*PI/m;
		
        repeatTimer = [NSTimer scheduledTimerWithTimeInterval:0.02
														target:self
													  selector:@selector(drawMotion:)
													  userInfo:nil
													   repeats:YES
						];
		[NSBezierPath setDefaultLineWidth:1];
	}
	return self;
}


- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void)drawMotion:(id)timer
{
	double a[2],y[2],u[2];
	int j;
	
	y[0]=x[0]; y[1]=x[1];
	
    [self lockFocus];
	
	[[NSColor blueColor] set];
	for (j = 0; j < m; j++){
		t = h * j;
		y[0]=x[0]; y[1]=x[1];
		a[0]=sfactor[0]*(y[0]-xm[0])+wm[0];
		a[1]=sfactor[1]*(y[1]-ym[0])+hm[0];
		
		runge(2, h, x, t);
		
		u[0]=sfactor[0]*(x[0]-xm[0])+wm[0];
		u[1]=sfactor[1]*(x[1]-ym[0])+hm[0];
		if(poi==1){
			[NSBezierPath strokeLineFromPoint:NSMakePoint(a[0], a[1]) toPoint:NSMakePoint(u[0], u[1])];}
	}	
	
	u[0]=sfactor[0]*(x[0]-xm[0])+wm[0];
	u[1]=sfactor[1]*(x[1]-ym[0])+hm[0];
	
	[[NSColor redColor] set];
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(u[0]-3, u[1]-3, 6, 6)] fill];
	
    [self unlockFocus];
    [[self window] flushWindow];
}

void runge(int n, double h, double x[], double t)
{
	double k1[n],k2[n],k3[n],k4[n],xtemp[n];
	int i;
	
	func(x,k1,t);
	for (i=0; i<n; i++) xtemp[i]=x[i]+k1[i]/2.0*h;
	t+=h/2.0;
	func(xtemp,k2,t);
	for (i=0; i<n; i++) xtemp[i]=x[i]+k2[i]/2.0*h;
	func(xtemp,k3,t);
	for (i=0; i<n; i++) xtemp[i]=x[i]+k3[i]*h;
	t+=h/2.0;
	func(xtemp,k4,t);
	for (i=0; i<n; i++) x[i]+=(k1[i]+2.0*(k2[i]+k3[i])+k4[i])/6.0*h;
}

void func(double x[], double f[], double t)
{
	
	f[0] = x[1];
	f[1] = -ak*x[1]-x[0]*x[0]*x[0] + 2.2*cos(t)+2.0;
}

- (void)drawRect:(NSRect)rect
{
	[[NSColor whiteColor] set];
	NSRectFill(rect);
	
	sfactor[0] = (rect.size.width-wm[0]-wm[1])/(xm[1]-xm[0]);
	sfactor[1] =(rect.size.height-hm[0]-hm[1])/(ym[1]-ym[0]);
	
	[self drawAxes];
}

- (void) mouseDown:(NSEvent *)theEvent
{
	NSPoint temp;
	NSPoint loc=[theEvent locationInWindow];
	
	loc.x -= [self frame].origin.x;
	loc.y -= [self frame].origin.y;
	
	x[0]=(loc.x-wm[0])/sfactor[0] + xm[0];
	x[1]=(loc.y-hm[0])/sfactor[1] + ym[0];	
	
	temp=[self convertPoint:loc toView:nil];
    [[NSColor greenColor] set];
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(temp.x-3, temp.y-3, 6, 6)] fill];
	
}

- (void)keyDown:(NSEvent *)theEvent
{
	NSString *keyChar=[theEvent charactersIgnoringModifiers];
	if ([keyChar isEqualToString:@"e"]){
		[self setNeedsDisplay:YES];
	}
	if ([keyChar isEqualToString:@"a"]){
		ak+=0.01;
	}
	if ([keyChar isEqualToString:@"s"]){
		ak-=0.01;
	}
	if ([keyChar isEqualToString:@"f"]){
		if(poi==1){poi=0;}else{poi=1;};
	}
}


- (void)drawAxes
{
	int ticx=5, ticy=5;
	int ix, iy, k, lt0, lt1, lt2;
	double temp, b[2];
	const CGFloat pat[2]={3.0,1.5};
	NSRect r;
	NSBezierPath *bp2;
	NSAffineTransform *transform=[NSAffineTransform transform];//4lines have to be added
	
	b[0]=sfactor[0]*(xm[1]-xm[0]);
	b[1]=sfactor[1]*(ym[1]-ym[0]);
	
	[[NSColor blueColor] set];	
	bp=[NSBezierPath bezierPath];
	[transform translateXBy:0.5f yBy:0.5f];//4lines have to be added
	[bp setLineWidth:1.0];
	bp2=[NSBezierPath bezierPath];
	[bp2 setLineWidth:0.5];
	
	//tics and labels for x-axis
	lt0=hm[0];
	lt1=lt0+ticy;
	lt2=lt0+ticy*2;
	temp=xm[0];
	k=0;
	while ((temp- (xm[1]+0.01*la[0]))<=0.0){
		ix=wm[0]+sfactor[0]*(temp-xm[0]);
		if(k%lav[0]==0 ){
			if(temp<xm[1]-0.1*la[0] && temp>xm[0]+0.1*la[0]){
				[bp moveToPoint:NSMakePoint(ix,lt0)];
				[bp lineToPoint:NSMakePoint(ix,lt2)];
				if(k!=0){
					[bp2 setLineDash:pat count:2 phase:0];
					[bp2 moveToPoint:NSMakePoint(ix,lt2)];
					[bp2 lineToPoint:NSMakePoint(ix, hm[0]+b[1])];
				}
			}
			[self drawMoji:temp xloc:ix yloc:lt0 hvmode:0];
			k=0;
		} else{
			if(temp<xm[1]-0.1*la[0] && temp>xm[0]+0.1*la[0]){
				[bp moveToPoint:NSMakePoint(ix, lt0)];
				[bp lineToPoint:NSMakePoint(ix, lt1)];
			}
		}
		temp +=la[0];
		k++;
	}
	//	[bp stroke];
	
	//tics and labels for y-axis
	lt0=wm[0];
	lt1=lt0+ticx;
	lt2=lt0+ticx*2;
	temp=ym[0];
	k=0;
	while ((temp- (ym[1]+0.01*la[1]))<=0.0){
		iy=hm[0]+sfactor[1]*(temp-ym[0]);
		if(k%lav[1]==0){
			if(temp<ym[1]-0.1*la[1] && temp>ym[0]+0.1*la[1]){
				[bp moveToPoint:NSMakePoint(lt0,iy)];
				[bp lineToPoint:NSMakePoint(lt2,iy)];
				if(k!=0){
					[bp2 setLineDash:pat count:2 phase:0];
					[bp2 moveToPoint:NSMakePoint(lt2,iy)];
					[bp2 lineToPoint:NSMakePoint(wm[0]+b[0],iy)];
				}
			}
			[self drawMoji:temp xloc:lt0 yloc:iy hvmode:1];
			k=0;
		} else{
			if( temp<ym[1]-0.1*la[1] && temp>ym[0]+0.1*la[1]){
				[bp moveToPoint:NSMakePoint(lt0,iy)];
				[bp lineToPoint:NSMakePoint(lt1,iy)];
			}
		}
		temp +=la[1];
		k++;
	}
	[bp transformUsingAffineTransform:transform];//4lines have to be added
	[bp stroke];
	[bp2 transformUsingAffineTransform:transform];//4lines have to be added
	[bp2 stroke];
	
	r=NSMakeRect(wm[0], hm[0], b[0], b[1]);	
	[[NSColor blueColor] set];
	bp=[NSBezierPath bezierPathWithRect:r];
	[bp transformUsingAffineTransform:transform];//4lines have to be added
	[bp stroke];
}

void gfcvt(double val, int ndigit,char s[])
{
	char str[31];
	int  i = 0,j = 0,dec,ndec,sign,ndig;
	int  fi[2];
	
	fi[0]=0; fi[1]=1;//temporary setting
	
	if (fabs(val)*pow(10.0,(double)ndigit) < .9) {
		if(fi[0]==0 | fi[1]==0){
			s[0] = '0'; s[1] = '\0';
			return;
		} else {
			s[0] = '0'; s[1] = '.';
			s[2] = '0'; s[3] = '\0';
			return;
		}
	}
	strcpy(str,fcvt(val,ndigit,&dec,&sign));
	if (sign == 1) s[i++] = '-';
	if (dec <= 0) s[i++] = '0';
	if (dec < 0) {
		s[i++] = '.';
		ndec = dec;
		ndig = ndigit;
		while (((ndec++) != 0)&&((ndig--) != 0)) s[i++] = '0';
	}
	do {
		if ((j == dec)&&(ndigit != 0)) s[i++] = '.';
	} while ((s[i++] = str[j++]) != '\0');
}

- (void)drawMoji:(double)num xloc:(double)xpos yloc:(double)ypos hvmode:(int)hv
{
//	char	string[31];
	unsigned int strlength;	
	NSString *str;
	NSMutableDictionary * attribs=[NSMutableDictionary dictionary];	
	
	[attribs setObject:[NSFont fontWithName:@"Helvetica" size:14]
				forKey:NSFontAttributeName];
	[attribs setObject:[NSColor blueColor]
				forKey:NSForegroundColorAttributeName];
	
//	gfcvt(num,1, string);
//	str=[NSString stringWithCString:string encoding:NSUTF8StringEncoding];
    str=[NSString stringWithFormat:@"%.1f", num];  //changed from the earlier version
	strlength=[str length];
	if(hv==0){
		[str drawAtPoint:NSMakePoint(xpos-strlength*4+2,ypos-24) withAttributes:attribs];	
	}else{
		[str drawAtPoint:NSMakePoint(xpos-strlength*4-14,ypos-8) withAttributes:attribs];
	}
}

@end
